function res = times(a,b)
% res = times(FT, x)
%

if a.adjoint % FT' performs inverse Fourier transform
    res = ifftnc(b);
    switch a.mode
        case 1
            res = res.*conj(a.ph);  % phase correction
            res = real(res);
    end
    
    if (~isempty(a.imsize)) % data size --> im size
        res_im = zeros(a.imsize);
    
        cr   = a.datsize;
        cp   = [a.imsize]/2;      % Center (Fourier DC point)
        stpt = round(cp-cr/2+1);      %
        enpt = round(cp+cr/2);      %
        
        if (sum(a.imsize > 1) == 1)     % 1D
            stpt = max(1,stpt);     
            enpt = min(a.imsize,enpt);   
            
            res_im(stpt(1):enpt(1)) = res;
        elseif (sum(a.imsize > 1) == 2) % 2D
            stpt = max([1 1],stpt);     
            enpt = min(a.imsize,enpt);   
            
            res_im(stpt(1):enpt(1),stpt(2):enpt(2)) = res;
        elseif (sum(a.imsize > 1) == 3) % 3D
            stpt = max([1 1 1],stpt);     
            enpt = min(a.imsize,enpt);   
            
            res_im(stpt(1):enpt(1),stpt(2):enpt(2),stpt(3):enpt(3)) = res;
        end
        res = res_im;
    end
else         % FT performs Fourier transform
    if (~isempty(a.imsize)) % im size --> data size
        cr   = a.datsize;
        cp   = [a.imsize]/2;      % Center (Fourier DC point)
        stpt = round(cp-cr/2+1);      
        enpt = round(cp+cr/2);      
        
        if (sum(a.imsize > 1) == 1)     % 1D
            stpt = max(1,stpt);     
            enpt = min(a.imsize,enpt);   
            
            b_dat = b(stpt(1):enpt(1));
        elseif (sum(a.imsize > 1) == 2) % 2D
            stpt = max([1 1],stpt);     
            enpt = min(a.imsize,enpt);   
            
            b_dat = b(stpt(1):enpt(1),stpt(2):enpt(2));
        elseif (sum(a.imsize > 1) == 3) % 3D
            stpt = max([1 1 1],stpt);     
            enpt = min(a.imsize,enpt);   
            
            b_dat = b(stpt(1):enpt(1),stpt(2):enpt(2),stpt(3):enpt(3));
        end
        b = b_dat;
    end
    
    switch a.mode
        case 1
            b = real(b);
            b = b.*a.ph;          % phase correction
    end
    res = fftnc(b);
    res = res.*a.mask;
end
