% This script demonstrates an overall example of LCAMP reconstruction
%
clear all; close all; clc;

load demo_lcamp2D.mat
addpath(strcat(pwd,'/utils'));
addpath(strcat(pwd,'/waveletcdf97')); 
figure_detail = 1;

[nx, ny] = size(x0);

% Partial Fourier Transform
Phi = pDFT(mask,1,1);

% Sparsifying Transform (Wavelet)
level      = 5;
Psi        = WaveletCDF97(level);

% Generate undersampled data
y = Phi*x0;

% Compute non-zero location based on wavelet denoising...
w0 = Psi*x0;
scale_fact  = 2;
sigma_hat   = median(abs(w0(:)))/.6745;
locMask     = abs(w0) > scale_fact*sqrt(2*log10(nx*ny))*sigma_hat;

% Determine the number of non-zero coefficients
numLoc = find(locMask);
numNZ  = round(length(numLoc));
rfact  = prod([nx ny])/sum(mask(:));

% Initial values
wn  = zeros(nx,ny);
zn  = y - Phi*(Psi'*wn);

abc = 1;    d = 0;
niter = 40;

% Compute first derivatives
pp  = real(locMask) | imag(locMask);
cnm = sum(pp(:))/numel(wn(:));

residual = zn;

res_norm = zeros(1,niter+1);
nn       = zeros(1,niter+1);

res_norm(1) = norm(zn(:));
norm_y      = norm(y(:));
nn(1)       = res_norm(1)/norm_y;

% Run LCAMP!!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
while abc < niter+1
    
    % Location Constrained First-order Approximate Message Passing
    zn = residual + rfact*zn*cnm;
    temp_z = Psi*(Phi'*zn) + wn;
    
    % Apply Location Constraint
    wn1 = (temp_z).*locMask;
    
    % Update the residual term
    residual = y - Phi*(Psi'*wn1);
    
    if (figure_detail == 1)
        figure(101);
        subplot(231); imshow(abs(wn),[0 .25]); title('w_n');
        subplot(232); imshow(abs(locMask)); title('LC');
        xlabel(sprintf('iter # = %d',abc));
        subplot(233); imshow(abs(wn1),[0 .25]); title('w_{n+1}');
        
        subplot(234); imshow(abs(residual),[0 .25]); title('Residual');
        subplot(236); imshow(abs(Psi'*wn1),[0 .75]); title('x0_{hat}');
        pause(0.5);
    end
    
    % Update Estimation
    wn  = wn1;
    abc = abc + 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
x0est = Psi'*wn;

% Compute normalized root mean square errors
max_x0  = max(abs(x0(:)));
sqrt_n  = sqrt(nx*ny);
nRMSE = 1/(max_x0*sqrt_n)*norm(x0(:) - x0est(:));

disp(   '=============================');
fprintf(' LCAMP Error      : %0.5g\n',nRMSE);
disp(   '=============================');

%% Figure part
zx = 161:221;
zy = 145:205;

figure(1); 
subplot(211);
imshow(abs([x0 x0est])); 
title('Original     LCAMP');
xlabel(sprintf('R_{net} is %.1f',rfact));
subplot(212);
imshow(abs([x0(zx,zy) x0est(zx,zy)])); 
title('Original     LCAMP');
