% This script demonstrates comparison between LCAMP and AMP
%
clear all; clc; close all;

load demo_lcamp2D.mat
addpath(strcat(pwd,'/utils'));
addpath(strcat(pwd,'/waveletcdf97'));

[nx, ny] = size(x0);

% Partial Fourier Transform
Phi = pDFT(mask,1,1);

% Sparsifying Transform (Wavelet)
level = 5;
Psi = WaveletCDF97(level);

% Generate undersampled data
y = Phi*x0;

% Zero-filling
x_zf = Phi'*y;

R = prod([nx ny])/sum(mask(:));
fprintf('Acceleration factor is %.1f\n',R);

%% Run LCAMP
opts.verbose = 1;
opts.randshift = 1;
opts.tol =  1e-5;
opts.maxIter = 100;
opts.lc = real(x0);

% Run LCAMP!!
x_est_lcamp = lcamp2d(Phi,Psi,y,opts);

% Compute normalized root mean square errors
max_x  = max(abs(x0(:)));
nRMSE_lcamp = 1/(max_x*sqrt((nx*ny)))*norm(x0(:) - x_est_lcamp(:));

%% Run AMP
opts.x0 = zeros(nx,ny);
opts.verbose = 1;
opts.randshift  = 1;
opts.tol =  1e-5;
opts.maxIter = 100;
opts.sigmaType = 1;

% Run AMP!!
x_est_amp = amp2d(Phi,Psi,y,opts);

% Compute normalized root mean square errors
max_x  = max(abs(x0(:)));
nRMSE_amp = 1/(max_x*sqrt((nx*ny)))*norm(x0(:) - x_est_amp(:));

%% Figure & Diplay RMSE
disp(   '=============================');
fprintf(' LCAMP Error      : %0.5g\n',nRMSE_lcamp);
fprintf(' AMP Error        : %0.5g\n',nRMSE_amp);
disp(   '=============================');

zx = 161:221;
zy = 145:205;

figure(1); 
subplot(211);
imshow(abs([x0 x_zf x_est_amp x_est_lcamp])); 
title('Original     Zero-Filling      AMP       LCAMP');
xlabel(sprintf('R_{net} is %.1f',R));
subplot(212);
imshow(abs([x0(zx,zy) x_zf(zx,zy) x_est_amp(zx,zy) x_est_lcamp(zx,zy)])); 
title('Original     Zero-Filling      AMP       LCAMP');
